@@ -12,23 +12,24 @@ use Ref::Util qw( is_blessed_ref );
 use Carp ();
 
 # ABSTRACT: Wasmtime linker class
-our $VERSION = '0.05'; # VERSION
+our $VERSION = '0.06'; # VERSION
 
 
 $ffi_prefix = 'wasmtime_linker_';
-$ffi->type('opaque' => 'wasmtime_linker_t');
+$ffi->load_custom_type('::PtrObject' => 'wasmtime_linker_t' => __PACKAGE__);
 
 
 $ffi->attach( new => ['wasm_store_t'] => 'wasmtime_linker_t' => sub {
   my($xsub, $class, $store) = @_;
-  my $ptr = $xsub->($store->{ptr});
-  bless { ptr => $ptr, store => $store }, $class;
+  my $self = $xsub->($store);
+  $self->{store} = $store;
+  $self;
 });
 
 
 $ffi->attach( allow_shadowing => [ 'wasmtime_linker_t', 'bool' ] => sub {
   my($xsub, $self, $value) = @_;
-  $xsub->($self->{ptr}, $value);
+  $xsub->($self, $value);
   $self;
 });
 
@@ -55,7 +56,7 @@ if(Wasm::Wasmtime::Error->can('new'))
       Carp::croak("not an extern: $extern");
     }
 
-    my $error = $xsub->($self->{ptr}, $module, $name, $extern->{ptr});
+    my $error = $xsub->($self, $module, $name, $extern);
     Carp::croak($error->message) if $error;
     $self;
   });
@@ -82,7 +83,7 @@ else
       Carp::croak("not an extern: $extern");
     }
 
-    my $ret = $xsub->($self->{ptr}, $module, $name, $extern->{ptr});
+    my $ret = $xsub->($self, $module, $name, $extern);
     unless($ret)
     {
       Carp::croak("Unknown error in define");
@@ -97,7 +98,7 @@ if(Wasm::Wasmtime::Error->can('new'))
 {
   $ffi->attach( define_wasi => ['wasmtime_linker_t', 'wasi_instance_t'] => 'wasmtime_error_t' => sub {
     my($xsub, $self, $wasi) = @_;
-    my $error = $xsub->($self->{ptr}, $wasi);
+    my $error = $xsub->($self, $wasi);
     Carp::croak($error->message) if $error;
     $self;
   });
@@ -106,7 +107,7 @@ else
 {
   $ffi->attach( define_wasi => ['wasmtime_linker_t', 'wasi_instance_t'] => 'bool' => sub {
     my($xsub, $self, $wasi) = @_;
-    my $ret = $xsub->($self->{ptr}, $wasi);
+    my $ret = $xsub->($self, $wasi);
     Carp::croak("Unknown error in define_wasi") unless $ret;
     $self;
   });
@@ -118,7 +119,7 @@ if(Wasm::Wasmtime::Error->can('new'))
   $ffi->attach( define_instance => ['wasmtime_linker_t', 'wasm_byte_vec_t*', 'wasm_instance_t'] => 'wasmtime_error_t' => sub {
     my($xsub, $self, $name, $instance) = @_;
     my $vname = Wasm::Wasmtime::ByteVec->new($name);
-    my $error = $xsub->($self->{ptr}, $vname, $instance->{ptr});
+    my $error = $xsub->($self, $vname, $instance);
     Carp::croak($error->message) if $error;
     $self;
   });
@@ -128,7 +129,7 @@ else
   $ffi->attach( define_instance => ['wasmtime_linker_t', 'wasm_byte_vec_t*', 'wasm_instance_t'] => 'bool' => sub {
     my($xsub, $self, $name, $instance) = @_;
     my $vname = Wasm::Wasmtime::ByteVec->new($name);
-    my $ret = $xsub->($self->{ptr}, $vname, $instance->{ptr});
+    my $ret = $xsub->($self, $vname, $instance);
     Carp::croak("Unknown error in define_instance") unless $ret;
     $self;
   });
@@ -137,11 +138,11 @@ else
 
 if(Wasm::Wasmtime::Error->can('new'))
 {
-  $ffi->attach( instantiate => ['wasmtime_linker_t','wasm_module_t','wasm_instance_t*','wasm_trap_t*'] => 'wasmtime_error_t' => sub {
+  $ffi->attach( instantiate => ['wasmtime_linker_t','wasm_module_t','opaque*','opaque*'] => 'wasmtime_error_t' => sub {
     my($xsub, $self, $module) = @_;
     my $trap;
     my $ptr;
-    my $error = $xsub->($self->{ptr}, $module->{ptr}, \$ptr, \$trap);
+    my $error = $xsub->($self, $module, \$ptr, \$trap);
     Carp::croak($error->message) if $error;
     if($trap)
     {
@@ -162,10 +163,10 @@ if(Wasm::Wasmtime::Error->can('new'))
 }
 else
 {
-  $ffi->attach( instantiate => ['wasmtime_linker_t','wasm_module_t','wasm_trap_t*' ] => 'wasm_instance_t' => sub {
+  $ffi->attach( instantiate => ['wasmtime_linker_t','wasm_module_t','opaque*' ] => 'opaque' => sub {
     my($xsub, $self, $module) = @_;
     my $trap;
-    my $ptr = $xsub->($self->{ptr}, $module->{ptr}, \$trap);
+    my $ptr = $xsub->($self, $module, \$trap);
     if($trap)
     {
       $trap = Wasm::Wasmtime::Trap->new($trap);
@@ -184,10 +185,7 @@ else
   });
 }
 
-$ffi->attach( [ 'delete' => 'DESTROY' ] => ['wasmtime_linker_t'] => sub {
-  my($xsub, $self) = @_;
-  $xsub->($self->{ptr}) if $self->{ptr};
-});
+_generate_destroy();
 
 1;
 
@@ -203,7 +201,7 @@ Wasm::Wasmtime::Linker - Wasmtime linker class
 
 =head1 VERSION
 
-version 0.05
+version 0.06
 
 =head1 SYNOPSIS
 
