@@ -8,61 +8,61 @@ use Wasm::Wasmtime::Store;
 use Wasm::Wasmtime::MemoryType;
 
 # ABSTRACT: Wasmtime memory class
-our $VERSION = '0.05'; # VERSION
+our $VERSION = '0.06'; # VERSION
 
 
 $ffi_prefix = 'wasm_memory_';
-$ffi->type('opaque' => 'wasm_memory_t');
+$ffi->load_custom_type('::PtrObject' => 'wasm_memory_t' => __PACKAGE__);
 
 
 $ffi->attach( new => ['wasm_store_t', 'wasm_memorytype_t'] => 'wasm_memory_t' => sub {
   my $xsub = shift;
   my $class = shift;
-  my $ptr;
-  my $owner;
   if(is_ref $_[0])
   {
     my($store, $memorytype) = @_;
-    $ptr = $xsub->($store->{ptr}, $memorytype->{ptr});
+    return $xsub->($store, $memorytype);
   }
   else
   {
-    ($ptr, $owner) = @_;
+    my($ptr, $owner) = @_;
+    return bless {
+      ptr   => $ptr,
+      owner => $owner,
+    }, $class;
   }
-  bless {
-    ptr   => $ptr,
-    owner => $owner,
-  }, $class;
 });
 
 
 $ffi->attach( type => ['wasm_memory_t'] => 'wasm_memorytype_t' => sub {
   my($xsub, $self) = @_;
-  Wasm::Wasmtime::MemoryType->new($xsub->($self->{ptr}), $self->{owner} || $self);
+  my $type = $xsub->($self);
+  $type->{owner} = $self->{owner} || $self if $type;
+  $type;
 });
 
 
 $ffi->attach( data => ['wasm_memory_t'] => 'opaque' => sub {
   my($xsub, $self) = @_;
-  $xsub->($self->{ptr});
+  $xsub->($self);
 });
 
 
 $ffi->attach( data_size => ['wasm_memory_t'] => 'size_t' => sub {
   my($xsub, $self) = @_;
-  $xsub->($self->{ptr});
+  $xsub->($self);
 });
 
 
 $ffi->attach( size => ['wasm_memory_t'] => 'uint32' => sub {
   my($xsub, $self) = @_;
-  $xsub->($self->{ptr});
+  $xsub->($self);
 });
 
 
 $ffi->attach( grow => ['wasm_memory_t', 'uint32'] => 'bool' => sub {
   my($xsub, $self, $delta) = @_;
-  $xsub->($self->{ptr}, $delta);
+  $xsub->($self, $delta);
 });
 
 
@@ -70,17 +70,11 @@ $ffi->attach( grow => ['wasm_memory_t', 'uint32'] => 'bool' => sub {
 $ffi->attach( as_extern => ['wasm_memory_t'] => 'opaque' => sub {
   my($xsub, $self) = @_;
   require Wasm::Wasmtime::Extern;
-  my $ptr = $xsub->($self->{ptr});
+  my $ptr = $xsub->($self);
   Wasm::Wasmtime::Extern->new($ptr, $self->{owner} || $self);
 });
 
-$ffi->attach( [ delete => "DESTROY" ] => ['wasm_memory_t'] => sub {
-  my($xsub, $self) = @_;
-  if(defined $self->{ptr} && !defined $self->{owner})
-  {
-    $xsub->($self->{ptr});
-  }
-});
+_generate_destroy();
 
 1;
 
@@ -96,7 +90,7 @@ Wasm::Wasmtime::Memory - Wasmtime memory class
 
 =head1 VERSION
 
-version 0.05
+version 0.06
 
 =head1 SYNOPSIS
 
