@@ -6,11 +6,11 @@ use Wasm::Wasmtime::FFI;
 use Wasm::Wasmtime::Store;
 
 # ABSTRACT: Wasmtime trap class
-our $VERSION = '0.05'; # VERSION
+our $VERSION = '0.06'; # VERSION
 
 
 $ffi_prefix = 'wasm_trap_';
-$ffi->type('opaque' => 'wasm_trap_t');
+$ffi->load_custom_type('::PtrObject' => 'wasm_trap_t' => __PACKAGE__);
 
 
 $ffi->attach( new => [ 'wasm_store_t', 'wasm_byte_vec_t*' ] => 'wasm_trap_t' => sub {
@@ -18,18 +18,16 @@ $ffi->attach( new => [ 'wasm_store_t', 'wasm_byte_vec_t*' ] => 'wasm_trap_t' =>
   my $class = shift;
   if(@_ == 1)
   {
-    my $pointer = shift;
+    my $ptr = shift;
     return bless {
-      ptr => $pointer,
+      ptr => $ptr,
     }, $class;
   }
   else
   {
     my $store = shift;
     my $message = Wasm::Wasmtime::ByteVec->new($_[0]);
-    return bless {
-      ptr => $xsub->($store->{ptr}, $message),
-    }, $class;
+    return $xsub->($store, $message);
   }
 });
 
@@ -37,17 +35,14 @@ $ffi->attach( new => [ 'wasm_store_t', 'wasm_byte_vec_t*' ] => 'wasm_trap_t' =>
 $ffi->attach( message => ['wasm_trap_t', 'wasm_byte_vec_t*'] => sub {
   my($xsub, $self) = @_;
   my $message = Wasm::Wasmtime::ByteVec->new;
-  $xsub->($self->{ptr}, $message);
+  $xsub->($self, $message);
   my $ret = $message->get;
   $ret =~ s/\0$//;
   $message->delete;
   $ret;
 });
 
-$ffi->attach( [ 'delete' => 'DESTROY' ] => ['wasm_trap_t'] => sub {
-  my($xsub, $self) = @_;
-  $xsub->($self->{ptr}) if $self->{ptr};
-});
+_generate_destroy();
 
 1;
 
@@ -63,7 +58,7 @@ Wasm::Wasmtime::Trap - Wasmtime trap class
 
 =head1 VERSION
 
-version 0.05
+version 0.06
 
 =head1 SYNOPSIS
 
